import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Main } from 'src/app/models/main.model';
import { MainService } from 'src/app/services/main.service';
import {FormControl} from '@angular/forms';
import {Observable} from 'rxjs';
import {map, startWith} from 'rxjs/operators';
import { NotifierService } from "angular-notifier";

@Component({
  selector: 'app-main-details',
  templateUrl: './main-details.component.html',
  styleUrls: ['./main-details.component.scss']
})
export class MainDetailsComponent implements OnInit {

  subjectsControl = new FormControl();
  subjects:string[] = [];
  subjectsfilteredOptions: Observable<string[]>;

  stateitemControl = new FormControl();
  stateitem:string[] = [];
  stateitemfilteredOptions: Observable<string[]>;

  statevalueControl = new FormControl();
  statevalue:string[] = [];
  statevaluefilteredOptions: Observable<string[]>;

  shallControl = new FormControl();
  shall:string[] = [];
  shallfilteredOptions: Observable<string[]>;

  verbControl = new FormControl();
  verb:string[] = [];
  verbfilteredOptions: Observable<string[]>;

  quantifierControl = new FormControl();
  quantifier:string[] = [];
  quantifierfilteredOptions: Observable<string[]>;

  numunitControl = new FormControl();
  numunit:string[] = [];
  numunitfilteredOptions: Observable<string[]>;

  fifuintsysstatsetControl = new FormControl();
  fifuintsysstatset:string[] = [];
  fifuintsysstatsetfilteredOptions: Observable<string[]>;

  stsysconsetControl = new FormControl();
  stsysconset:string[] = [];
  stsysconsetfilteredOptions: Observable<string[]>;

  public static id: any [] = [];
  private readonly notifier: NotifierService;
  mains?: Main[];

  currentmain: Main  = {
    sys_fun_inter: "",
    state_item_before_verb: "",
    statevalue_before_verb: "",
    shall: "",
    verb: "",
    quantifier: "",
    numerical: 0,
    mumerical_units: "",
    flow_function_interface_item_system_state_stateset: "",
    statevalue_system_connection_stateset: "",
  };
  message = '';

  constructor(
    private mainService: MainService,
    private route: ActivatedRoute,
    private router: Router, 
    notifierService: NotifierService) { 
    this.notifier = notifierService;
    this.subjectsfilteredOptions = this.subjectsControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.subjects, value))
    );
    this.stateitemfilteredOptions = this.stateitemControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.stateitem, value))
    ); 
    this.statevaluefilteredOptions = this.statevalueControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.statevalue, value))
    ); 
    this.shallfilteredOptions = this.shallControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.shall, value))
    );
    this.verbfilteredOptions = this.verbControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.verb, value))
    );
    this.quantifierfilteredOptions = this.quantifierControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.quantifier, value))
    );
    this.numunitfilteredOptions = this.numunitControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.numunit, value))
    ); 
    this.fifuintsysstatsetfilteredOptions = this.fifuintsysstatsetControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.fifuintsysstatset, value))
    );
    this.stsysconsetfilteredOptions = this.stsysconsetControl.valueChanges.pipe(
      startWith(""),
      map(value => this._filter(this.stsysconset, value))
    );
  }

  ngOnInit(): void {
    this.getchoices(); 
    this.getAllMain(this.route.snapshot.params.id);
  }


  private _filter(data:string[], value: string): string[] {
    const filterValue = value.toLowerCase();
    return data.filter(option => option.toLowerCase().indexOf(filterValue) === 0);
  }

  getAllMain(id: string): void {
    this.mainService.getAll(id)
      .subscribe(
        data => {
          MainDetailsComponent.id.length =0;
          this.mains = data;
          for (var val of data) {
            MainDetailsComponent.id.push(val.id);
            this.getMain(id, val.id);
          }
        },
        error => {
          console.log(error);
        });
  }
  getMain(main: any, id: string): void {
    //PrefixDetailsComponent.id = id;
    this.mainService.get(main, id)
      .subscribe(
        data => {
          this.currentmain = data;
          //console.log(data)
          this.subjectsControl.setValue(this.currentmain.sys_fun_inter);
          this.stateitemControl.setValue(this.currentmain.state_item_before_verb);
          this.statevalueControl.setValue(this.currentmain.statevalue_before_verb);
          this.shallControl.setValue(this.currentmain.shall);
          this.verbControl.setValue(this.currentmain.verb);
          this.quantifierControl.setValue(this.currentmain.quantifier);
          // num
          this.numunitControl.setValue(this.currentmain.mumerical_units);
          this.fifuintsysstatsetControl.setValue(this.currentmain.flow_function_interface_item_system_state_stateset);
          this.stsysconsetControl.setValue(this.currentmain.statevalue_system_connection_stateset);
        },
        error => {
          console.log(error);
        });
  }


  updateMain(): void {
    this.currentmain.sys_fun_inter = this.subjectsControl.value;
    this.currentmain.state_item_before_verb = this.stateitemControl.value;
    this.currentmain.statevalue_before_verb = this.statevalueControl.value;
    this.currentmain.shall = this.shallControl.value;
    this.currentmain.verb = this.verbControl.value;
    this.currentmain.quantifier = this.quantifierControl.value;
    //num
    this.currentmain.mumerical_units = this.numunitControl.value;
    this.currentmain.flow_function_interface_item_system_state_stateset = this.fifuintsysstatsetControl.value;
    this.currentmain.statevalue_system_connection_stateset = this.stsysconsetControl.value;
    for (let i=0; i<MainDetailsComponent.id.length; i++){
    this.mainService.update(this.route.snapshot.params.id, MainDetailsComponent.id[i], this.currentmain)
      .subscribe(
        response => {
          this.notifier.notify("success",JSON.stringify("Prefix updated"));
          this.message = response.message;
        },
        error => {
          console.log(error);
          this.notifier.notify("warning",(JSON.stringify(error["error"])));
        });}
  }


  getchoices(): void{
    this.mainService.getsubject()
      .subscribe(
        data => {
          for (let x=0; x < data.length; x++ ){
          this.subjects.push(data[x][1]);}
        },
        error => {
          console.log(error);
        });
    
    this.mainService.getstateitem()
    .subscribe(
      data => {
        //no data from State and Item
        console.log(data);
        for (let x=0; x < data.length; x++ ){
        this.stateitem.push(data[x][1]);}
      },
      error => {
        console.log(error);
      });
    this.mainService.getstatevalue()
    .subscribe(
      data => {
        for (let x=0; x < data.length; x++ ){
        this.statevalue.push(data[x][1]);}
      },
      error => {
        console.log(error);
      });
    this.mainService.getshall()
    .subscribe(
      data => {
        for (let x=0; x < data.length; x++ ){
        this.shall.push(data[x][1]);}
      },
      error => {
        console.log(error);
      });
    this.mainService.getverb()
    .subscribe(
      data => {
        for (let x=0; x < data.length; x++ ){
        this.verb.push(data[x][1]);}
      },
      error => {
        console.log(error);
      });
    this.mainService.getquantifier()
    .subscribe(
      data => {
        for (let x=0; x < data.length; x++ ){
        this.quantifier.push(data[x][1]);}
      },
      error => {
        console.log(error);
      });
      //num
    this.mainService.getnumunit()
    .subscribe(
      data => {
        for (let x=0; x < data.length; x++ ){
        this.numunit.push(data[x][1]);}
      },
      error => {
        console.log(error);
      });
    this.mainService.getfifuintsysstatset()
    .subscribe(
      data => {
        for (let x=0; x < data.length; x++ ){
        this.fifuintsysstatset.push(data[x][1]);}
      },
      error => {
        console.log(error);
      });
    this.mainService.getstsysconset()
    .subscribe(
      data => {
        for (let x=0; x < data.length; x++ ){
        this.stsysconset.push(data[x][1]);}
      },
      error => {
        console.log(error);
      });
    }
  }


